@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2015 BEA Systems, Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2005, 2018 BEA Systems, Inc. and others
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    mkaufman@bea.com - initial API and implementation
@@ -317,7 +320,7 @@ public class GeneratedFileManager
 	public GeneratedFileManager(final AptProject aptProject, final GeneratedSourceFolderManager gsfm) {
 		_jProject = aptProject.getJavaProject();
 		_gsfm = gsfm;
-		_buildDeps = new GeneratedFileMap(_jProject.getProject());
+		_buildDeps = new GeneratedFileMap(_jProject.getProject(), gsfm.isTestCode());
 		_clearDuringReconcile = new HashSet<>();
 		_reconcileDeps = new ManyToMany<>();
 		_reconcileNonDeps = new ManyToMany<>();
@@ -346,11 +349,13 @@ public class GeneratedFileManager
 	{
 		try {
 			// clear out any generated source folder config markers
-			IMarker[] markers = _jProject.getProject().findMarkers(AptPlugin.APT_CONFIG_PROBLEM_MARKER, true,
-					IResource.DEPTH_INFINITE);
-			if (markers != null) {
-				for (IMarker marker : markers)
-					marker.delete();
+			if(!_gsfm.isTestCode()) {
+				IMarker[] markers = _jProject.getProject().findMarkers(AptPlugin.APT_CONFIG_PROBLEM_MARKER, true,
+						IResource.DEPTH_INFINITE);
+				if (markers != null) {
+					for (IMarker marker : markers)
+						marker.delete();
+				}
 			}
 		} catch (CoreException e) {
 			AptPlugin.log(e, "Unable to delete configuration marker."); //$NON-NLS-1$
