@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2007, 2017 BEA Systems, Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *    wharley@bea.com - initial API and implementation
@@ -392,6 +395,9 @@ public class Factory {
 		if (binding instanceof SplitPackageBinding && binding.enclosingModule != null) {
 			binding = ((SplitPackageBinding) binding).getIncarnation(binding.enclosingModule);
 		}
+		if (binding == null) {
+			return null;
+		}
 		return new PackageElementImpl(_env, binding);
 	}
 	
