@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2006, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2006, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -1361,9 +1364,9 @@ public class EclipseFileManager implements StandardJavaFileManager {
 			Map<String, String> options = new HashMap<>();
 			// FIXME: Find a way to get the options from the EclipseCompiler and pass it to the parser.
 			// FIXME: need to be the latest and not hardcoded value
-			options.put(CompilerOptions.OPTION_Compliance, CompilerOptions.VERSION_10);
-			options.put(CompilerOptions.OPTION_Source, CompilerOptions.VERSION_10);
-			options.put(CompilerOptions.OPTION_TargetPlatform, CompilerOptions.VERSION_10);
+			options.put(CompilerOptions.OPTION_Compliance, CompilerOptions.VERSION_11);
+			options.put(CompilerOptions.OPTION_Source, CompilerOptions.VERSION_11);
+			options.put(CompilerOptions.OPTION_TargetPlatform, CompilerOptions.VERSION_11);
 			CompilerOptions compilerOptions = new CompilerOptions(options);
 			ProblemReporter problemReporter = 
 					new ProblemReporter(
@@ -1421,6 +1424,10 @@ public class EclipseFileManager implements StandardJavaFileManager {
 			LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.MODULE_SOURCE_PATH, moduleName);
 			deriveOutputLocationForModules(moduleName, wrapper.paths);
 			result = getLocationForModule(location, moduleName);
+		} else if (result == null && location == StandardLocation.SOURCE_OUTPUT) {
+			LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.MODULE_SOURCE_PATH, moduleName);
+			deriveSourceOutputLocationForModules(moduleName, wrapper.paths);
+			result = getLocationForModule(location, moduleName);
 		}
 		return result;
 	}
@@ -1502,6 +1509,28 @@ public class EclipseFileManager implements StandardJavaFileManager {
 			}
 		}
 	}
+	private void deriveSourceOutputLocationForModules(String moduleName, Collection<? extends Path> paths) {
+		LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT, moduleName);
+		if (wrapper == null) {
+			// First get from our internally known location for legacy/unnamed location
+			wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT, ""); //$NON-NLS-1$
+			if (wrapper == null) {
+				wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT);
+			}
+			if (wrapper != null) {
+				Iterator<? extends Path> iterator = wrapper.paths.iterator();
+				if (iterator.hasNext()) {
+					try {
+					// Per module output location is always a singleton list
+					Path path = iterator.next().resolve(moduleName);
+					this.locationHandler.setLocation(StandardLocation.SOURCE_OUTPUT, moduleName, Collections.singletonList(path));
+					} catch(Exception e) {
+						e.printStackTrace();
+					}
+				}
+			}
+		}
+	}
 	@Override
 	public void setLocationForModule(Location location, String moduleName, Collection<? extends Path> paths) throws IOException {
 		validateModuleLocation(location, moduleName);
