@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -115,8 +118,16 @@ public abstract class AssistParser extends Parser {
 
 	protected boolean isFirst = false;
 
-	public AssistParser snapShot;
-	private static final int[] RECOVERY_TOKENS = new int [] { TokenNameSEMICOLON, TokenNameRPAREN,};
+	/**
+	 * Each nested block may capture a snapshot at its start, which may be updated during commit().
+	 * {@link #snapShotPositions} is a matching stack of bodyStart positions.
+	 * Both stacks are indexed by their shared stack pointer {@link #snapShotPtr}.
+	 */
+	AssistParser[] snapShotStack = new AssistParser[3];
+	int[] snapShotPositions = new int[3];
+	int snapShotPtr = -1;
+
+	protected static final int[] RECOVERY_TOKENS = { TokenNameSEMICOLON, TokenNameRPAREN, TokenNameRBRACE, TokenNameRBRACKET};
 
 
 public AssistParser(ProblemReporter problemReporter) {
@@ -186,7 +197,7 @@ public RecoveredElement buildInitialRecoveryState(){
 		RecoveredElement element = super.buildInitialRecoveryState();
 		flushAssistState();
 		flushElementStack();
-		this.snapShot = null;
+		this.snapShotPtr = -1;
 		initModuleInfo(element);
 		return element;
 	}
@@ -510,7 +521,7 @@ protected boolean triggerRecoveryUponLambdaClosure(Statement statement, boolean
 							stackLength);
 				}
 				this.stack[this.stateStackTop] = this.unstackedAct;
-				commit();
+				commit(false);
 				this.stateStackTop --;
 			}
 			return false;
@@ -572,7 +583,8 @@ protected boolean triggerRecoveryUponLambdaClosure(Statement statement, boolean
 			}
 		}
 	}
-	this.snapShot = null;
+	if (this.snapShotPtr > -1)
+		popSnapShot();
 	return lambdaClosed;
 }
 public Statement replaceAssistStatement(RecoveredElement top, ASTNode assistParent, int start, int end, Statement stmt) {
@@ -632,6 +644,18 @@ protected void consumeBlockStatements() {
 	}
 }
 @Override
+protected void consumeBlock() {
+	super.consumeBlock();
+	if (this.snapShotPtr > -1) {
+		ASTNode top = this.astStack[this.astPtr];
+		if (top instanceof Block) {
+			// check positions for sanity:
+			assert this.snapShotPositions[this.snapShotPtr] == top.sourceStart : "Block positions should be consistent"; //$NON-NLS-1$
+			popSnapShot();
+		}
+	}
+}
+@Override
 protected void consumeFieldDeclaration() {
 	super.consumeFieldDeclaration();
 	if (triggerRecoveryUponLambdaClosure((Statement) this.astStack[this.astPtr], true)) {
@@ -682,6 +706,14 @@ protected void consumeMethodDeclaration(boolean isNotAbstract, boolean isDefault
 		popElement(K_METHOD_DELIMITER);
 	}
 	super.consumeMethodDeclaration(isNotAbstract, isDefaultMethod);
+	if (this.snapShotPtr > -1) {
+		ASTNode top = this.astStack[this.astPtr];
+		if (top instanceof AbstractMethodDeclaration) {
+			// check positions for sanity:
+			assert this.snapShotPositions[this.snapShotPtr] + 1 == ((AbstractMethodDeclaration) top).bodyStart : "Method positions should be consistent"; //$NON-NLS-1$
+			popSnapShot();
+		}
+	}
 }
 @Override
 protected void consumeMethodHeader() {
@@ -825,7 +857,7 @@ protected void consumeOpenBlock() {
 			}
 			this.stack[this.stateStackTop++] = this.unstackedAct; // transition to Block ::= OpenBlock  .LBRACE BlockStatementsopt RBRACE
 			this.stack[this.stateStackTop] = tAction(this.unstackedAct, this.currentToken); // transition to Block ::= OpenBlock LBRACE  .BlockStatementsopt RBRACE 
-			commit();
+			commit(true);
 			this.stateStackTop -= 2;
 		}
 	}
@@ -2198,11 +2230,32 @@ public void reset(){
 	flushAssistState();
 }
 
-protected void commit() {
-	if (this.snapShot == null) {
-		this.snapShot = createSnapShotParser();
+void commit(boolean isStart) {
+	int newSnapShotPosition = this.scanner.startPosition;
+	if (this.snapShotPtr == -1) {
+		// first commit:
+		addNewSnapShot(newSnapShotPosition);
+	} else {
+		// already have a snapshot, does it match the current position and can thus be reused?
+		int currentStartPosition = isStart ? newSnapShotPosition : this.blockStarts[this.realBlockPtr];
+		if (currentStartPosition != this.snapShotPositions[this.snapShotPtr])
+			addNewSnapShot(newSnapShotPosition); // no match, create a new one
 	}
-	this.snapShot.copyState(this);
+	this.snapShotStack[this.snapShotPtr].copyState(this);
+}
+
+void addNewSnapShot(int newSnapShotPosition) {
+	if (++this.snapShotPtr >= this.snapShotStack.length) {
+		int len = this.snapShotStack.length;
+		System.arraycopy(this.snapShotStack, 0, this.snapShotStack = new AssistParser[len+3], 0, len);
+		System.arraycopy(this.snapShotPositions, 0, this.snapShotPositions = new int[len+3], 0, len);
+	}
+	this.snapShotStack[this.snapShotPtr] = createSnapShotParser();
+	this.snapShotPositions[this.snapShotPtr] = newSnapShotPosition;
+}
+
+void popSnapShot() {
+	this.snapShotStack[this.snapShotPtr--] = null;
 }
 
 protected boolean assistNodeNeedsStacking() {
@@ -2230,9 +2283,12 @@ protected int fallBackToSpringForward(Statement unused) {
 			
 	// If triggered fake EOF at completion site, see if the real next token would have passed muster.
 	if (this.currentToken == TokenNameEOF) {
-		if (this.scanner.eofPosition < this.scanner.source.length) {
+		int extendedEnd = this.scanner.source.length;
+		if (this.referenceContext instanceof AbstractMethodDeclaration)
+			extendedEnd = ((AbstractMethodDeclaration) this.referenceContext).bodyEnd; // no use parsing beyond the method's body end
+		if (this.scanner.eofPosition < extendedEnd) {
 			shouldStackAssistNode();
-			this.scanner.eofPosition = this.scanner.source.length;
+			this.scanner.eofPosition = extendedEnd;
 			nextToken = getNextToken();
 			if (automatonWillShift(nextToken, automatonState)) {
 				this.currentToken = nextToken;
@@ -2256,10 +2312,10 @@ protected int fallBackToSpringForward(Statement unused) {
 		}
 	}
 	// OK, no in place resumption, no local repair, fast forward to next statement.
-	if (this.snapShot == null)
+	if (this.snapShotPtr == -1)
 		return RESTART;
 
-	this.copyState(this.snapShot);
+	this.copyState(this.snapShotStack[this.snapShotPtr]);
 	if (assistNodeNeedsStacking()) {
 		this.currentToken = TokenNameSEMICOLON;
 		return RESUME;
