@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -214,7 +217,18 @@
  *									RepeatableAnnotationIsDocumented
  *									RepeatableAnnotationIsInherited
  *									RepeatableAnnotationWithRepeatingContainerAnnotation
-*******************************************************************************/
+ *									VarLocalMultipleDeclarators
+ *									VarLocalCannotBeArray
+ *									VarLocalReferencesItself
+ *									VarLocalWithoutInitizalier
+ *									VarLocalInitializedToNull
+ *									VarLocalCannotBeArrayInitalizers
+ *									VarLocalCannotBeLambda 
+ *									VarLocalCannotBeMethodReference
+ *									VarIsReserved
+ *									VarIsReservedInFuture
+ *									VarIsNotAllowedHere
+******************************************************************************/
 package org.eclipse.jdt.core.compiler;
 
 import org.eclipse.jdt.internal.compiler.lookup.ProblemReasons;
@@ -1483,7 +1497,7 @@ void setSourceStart(int sourceStart);
     int InterfaceSuperInvocationNotBelow18 = Internal + Syntax + 667;
     /** @since 3.13*/
     int InterfaceStaticMethodInvocationNotBelow18 = Internal + Syntax + 668;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int FieldMustBeFinal = Internal + 669;
 
 
@@ -1782,7 +1796,9 @@ void setSourceStart(int sourceStart);
 	int RedundantNullCheckOnField = Internal + 941;
 	/** @since 3.10 */
 	int FieldComparisonYieldsFalse = Internal + 942;
-	
+	/** @since 3.14 */
+	int RedundantNullDefaultAnnotationModule = Internal + 943;
+
 	/** @since 3.10 */
 	int ArrayReferencePotentialNullReference = Internal + 951;
 	/** @since 3.10 */
@@ -1947,28 +1963,46 @@ void setSourceStart(int sourceStart);
 	/** @since 3.14 */
 	int OverridingTerminallyDeprecatedMethod = MethodRelated + 1404;
 	// with since
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingDeprecatedSinceVersionType = TypeRelated + 1405;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingDeprecatedSinceVersionMethod = MethodRelated + 1406;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingDeprecatedSinceVersionConstructor = MethodRelated + 1407;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingDeprecatedSinceVersionField = FieldRelated + 1408;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int OverridingDeprecatedSinceVersionMethod = MethodRelated + 1409;
 	// terminally with since
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingTerminallyDeprecatedSinceVersionType = TypeRelated + 1410;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingTerminallyDeprecatedSinceVersionMethod = MethodRelated + 1411;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingTerminallyDeprecatedSinceVersionConstructor = MethodRelated + 1412;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int UsingTerminallyDeprecatedSinceVersionField = FieldRelated + 1413;
-	/** @since 3.13 */
+	/** @since 3.14 */
 	int OverridingTerminallyDeprecatedSinceVersionMethod = MethodRelated + 1414;
 
+	// unused constants:
+	/** @since 3.14 */
+	int UsingDeprecatedPackage = ModuleRelated + 1425;
+	/** @since 3.14 */
+	int UsingDeprecatedSinceVersionPackage = ModuleRelated + 1426;
+	/** @since 3.14 */
+	int UsingTerminallyDeprecatedPackage = ModuleRelated + 1427;
+	/** @since 3.14 */
+	int UsingTerminallyDeprecatedSinceVersionPackage = ModuleRelated + 1428;
+	// deprecation of modules:
+	/** @since 3.14 */
+	int UsingDeprecatedModule = ModuleRelated + 1429;
+	/** @since 3.14 */
+	int UsingDeprecatedSinceVersionModule = ModuleRelated + 1430;
+	/** @since 3.14 */
+	int UsingTerminallyDeprecatedModule = ModuleRelated + 1431;
+	/** @since 3.14 */
+	int UsingTerminallyDeprecatedSinceVersionModule = ModuleRelated + 1432;
 	
 	/** @since 3.14 */
 	int NotAccessibleType = TypeRelated + 1450;
@@ -1992,6 +2026,8 @@ void setSourceStart(int sourceStart);
 	int MissingRequiresTransitiveForTypeInAPI = ModuleRelated + 1459;
 	/** @since  3.14 */
 	int UnnamedPackageInNamedModule = ModuleRelated + 1460;
+	/** @since  3.14 */
+	int UnstableAutoModuleName = ModuleRelated + 1461;
 
 	/** @since 3.13 */
 	int RedundantNullDefaultAnnotationLocal = Internal + 1062;
@@ -2012,4 +2048,33 @@ void setSourceStart(int sourceStart);
 	int UnlikelyCollectionMethodArgumentType = 1200;
 	/** @since 3.13 */
 	int UnlikelyEqualsArgumentType = 1201;
+
+	/* Local-Variable Type Inference */
+	/** @since 3.14 */
+	int VarLocalMultipleDeclarators = Syntax + 1500; // ''var'' is not allowed in a compound declaration
+	/** @since 3.14 */
+	int VarLocalCannotBeArray = Syntax + 1501; // ''var'' is not allowed as an element type of an array
+	/** @since 3.14 */
+	int VarLocalReferencesItself = Syntax + 1502; // Declaration using ''var'' may not contin references to itself
+	/** @since 3.14 */
+	int VarLocalWithoutInitizalier = Syntax + 1503; // Cannot use ''var'' on variable without initializer
+	/** @since 3.14 */
+	int VarLocalInitializedToNull = TypeRelated + 1504; // Variable initialized to ''null'' needs an explicit target-type
+	/** @since 3.14 */
+	int VarLocalInitializedToVoid = TypeRelated + 1505; // Variable initializer is ''void'' -- cannot infer variable type 
+	/** @since 3.14 */
+	int VarLocalCannotBeArrayInitalizers = TypeRelated + 1506; // Array initializer needs an explicit target-type
+	/** @since 3.14 */
+	int VarLocalCannotBeLambda = TypeRelated + 1507; // Lambda expression needs an explicit target-type 
+	/** @since 3.14 */
+	int VarLocalCannotBeMethodReference = TypeRelated + 1508; // Method reference needs an explicit target-type
+	/** @since 3.14 */
+	int VarIsReserved = Syntax + 1509; // ''var'' is not a valid type name
+	/** @since 3.14 */
+	int VarIsReservedInFuture = Syntax + 1510; // ''var'' should not be used as an type name, since it is a reserved word from source level 10 on
+	/** @since 3.14 */
+	int VarIsNotAllowedHere = Syntax + 1511; // ''var'' is not allowed here
+	/** @since 3.16 */
+	int VarCannotBeMixedWithNonVarParams = Syntax + 1512; // ''var'' cannot be mixed with explicit or implicit parameters
+	
 }
