@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -437,10 +440,15 @@ public class ReferenceExpression extends FunctionalExpression implements IPolyEx
 		
 		if (this.isConstructorReference()) {
 			ReferenceBinding allocatedType = codegenBinding.declaringClass;
-			if (codegenBinding.isPrivate() && TypeBinding.notEquals(enclosingSourceType, (allocatedType = codegenBinding.declaringClass))) {
+			if (codegenBinding.isPrivate() &&
+					TypeBinding.notEquals(enclosingSourceType, (allocatedType = codegenBinding.declaringClass))) {
 				if ((allocatedType.tagBits & TagBits.IsLocalType) != 0) {
 					codegenBinding.tagBits |= TagBits.ClearPrivateModifier;
 				} else {
+					if (currentScope.enclosingSourceType().isNestmateOf(this.binding.declaringClass)) {
+						this.syntheticAccessor = codegenBinding;
+						return;
+					}
 					this.syntheticAccessor = ((SourceTypeBinding) allocatedType).addSyntheticMethod(codegenBinding, false);
 					currentScope.problemReporter().needToEmulateMethodAccess(codegenBinding, this);
 				}
@@ -634,6 +642,9 @@ public class ReferenceExpression extends FunctionalExpression implements IPolyEx
 			}
 
 	    	if (this.expectedType == null && this.expressionContext == INVOCATION_CONTEXT) {
+	    		if (compilerOptions.isAnnotationBasedNullAnalysisEnabled && this.binding != null) {
+	    			ImplicitNullAnnotationVerifier.ensureNullnessIsKnown(this.binding, scope);
+	    		}
 	    		return new PolyTypeBinding(this);
 			}
 
