@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -30,12 +33,12 @@ public class MethodInfo extends ClassFileStruct implements IBinaryMethod, Compar
 	protected int accessFlags;
 	protected int attributeBytes;
 	protected char[] descriptor;
-	protected char[][] exceptionNames;
+	protected volatile char[][] exceptionNames;
 	protected char[] name;
 	protected char[] signature;
 	protected int signatureUtf8Offset;
 	protected long tagBits;
-	protected char[][] argumentNames;
+	protected volatile char[][] argumentNames;
 	protected long version;
 
 public static MethodInfo createMethod(byte classFileBytes[], int offsets[], int offset, long version) {
@@ -226,6 +229,7 @@ protected MethodInfo (byte classFileBytes[], int offsets[], int offset, long ver
 	this.signatureUtf8Offset = -1;
 	this.version = version;
 }
+@Override
 public int compareTo(Object o) {
 	MethodInfo otherMethod = (MethodInfo) o;
 	int result = new String(getSelector()).compareTo(new String(otherMethod.getSelector()));
@@ -245,38 +249,34 @@ public boolean equals(Object o) {
 public int hashCode() {
 	return CharOperation.hashCode(getSelector()) + CharOperation.hashCode(getMethodDescriptor());
 }
-/**
- * @return the annotations or null if there is none.
- */
+
+@Override
 public IBinaryAnnotation[] getAnnotations() {
 	return null;
 }
 /**
  * @see org.eclipse.jdt.internal.compiler.env.IGenericMethod#getArgumentNames()
  */
+@Override
 public char[][] getArgumentNames() {
 	if (this.argumentNames == null) {
 		readCodeAttribute();
 	}
 	return this.argumentNames;
 }
+@Override
 public Object getDefaultValue() {
 	return null;
 }
-/**
- * Answer the resolved names of the exception types in the
- * class file format as specified in section 4.2 of the Java 2 VM spec
- * or null if the array is empty.
- *
- * For example, java.lang.String is java/lang/String.
- * @return char[][]
- */
+
+@Override
 public char[][] getExceptionTypeNames() {
 	if (this.exceptionNames == null) {
 		readExceptionAttributes();
 	}
 	return this.exceptionNames;
 }
+@Override
 public char[] getGenericSignature() {
 	if (this.signatureUtf8Offset != -1) {
 		if (this.signature == null) {
@@ -287,15 +287,8 @@ public char[] getGenericSignature() {
 	}
 	return null;
 }
-/**
- * Answer the receiver's method descriptor which describes the parameter &
- * return types as specified in section 4.3.3 of the Java 2 VM spec.
- *
- * For example:
- *   - int foo(String) is (Ljava/lang/String;)I
- *   - void foo(Object[]) is (I)[Ljava/lang/Object;
- * @return char[]
- */
+
+@Override
 public char[] getMethodDescriptor() {
 	if (this.descriptor == null) {
 		// read the name
@@ -310,29 +303,28 @@ public char[] getMethodDescriptor() {
  * Set the AccDeprecated and AccSynthetic bits if necessary
  * @return int
  */
+@Override
 public int getModifiers() {
 	if (this.accessFlags == -1) {
 		// compute the accessflag. Don't forget the deprecated attribute
-		this.accessFlags = u2At(0);
 		readModifierRelatedAttributes();
 	}
 	return this.accessFlags;
 }
+@Override
 public IBinaryAnnotation[] getParameterAnnotations(int index, char[] classFileName) {
 	return null;
 }
+@Override
 public int getAnnotatedParametersCount() {
 	return 0;
 }
+@Override
 public IBinaryTypeAnnotation[] getTypeAnnotations() {
 	return null;
 }
-/**
- * Answer the name of the method.
- *
- * For a constructor, answer <init> & <clinit> for a clinit method.
- * @return char[]
- */
+
+@Override
 public char[] getSelector() {
 	if (this.name == null) {
 		// read the name
@@ -341,6 +333,7 @@ public char[] getSelector() {
 	}
 	return this.name;
 }
+@Override
 public long getTagBits() {
 	return this.tagBits;
 }
@@ -361,6 +354,7 @@ protected void initialize() {
  * Answer true if the method is a class initializer, false otherwise.
  * @return boolean
  */
+@Override
 public boolean isClinit() {
 	return org.eclipse.jdt.internal.compiler.classfmt.JavaBinaryNames.isClinit(getSelector());
 }
@@ -368,6 +362,7 @@ public boolean isClinit() {
  * Answer true if the method is a constructor, false otherwise.
  * @return boolean
  */
+@Override
 public boolean isConstructor() {
 	return org.eclipse.jdt.internal.compiler.classfmt.JavaBinaryNames.isConstructor(getSelector());
 }
@@ -378,9 +373,10 @@ public boolean isConstructor() {
 public boolean isSynthetic() {
 	return (getModifiers() & ClassFileConstants.AccSynthetic) != 0;
 }
-private void readExceptionAttributes() {
+private synchronized void readExceptionAttributes() {
 	int attributesCount = u2At(6);
 	int readOffset = 8;
+	char[][] names = null;
 	for (int i = 0; i < attributesCount; i++) {
 		int utf8Offset = this.constantPoolOffsets[u2At(readOffset)] - this.structOffset;
 		char[] attributeName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
@@ -390,15 +386,15 @@ private void readExceptionAttributes() {
 			// place the readOffset at the beginning of the exceptions table
 			readOffset += 8;
 			if (entriesNumber == 0) {
-				this.exceptionNames = noException;
+				names = noException;
 			} else {
-				this.exceptionNames = new char[entriesNumber][];
+				names = new char[entriesNumber][];
 				for (int j = 0; j < entriesNumber; j++) {
 					utf8Offset =
 						this.constantPoolOffsets[u2At(
 							this.constantPoolOffsets[u2At(readOffset)] - this.structOffset + 1)]
 							- this.structOffset;
-					this.exceptionNames[j] = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
+					names[j] = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
 					readOffset += 2;
 				}
 			}
@@ -406,11 +402,14 @@ private void readExceptionAttributes() {
 			readOffset += (6 + u4At(readOffset + 2));
 		}
 	}
-	if (this.exceptionNames == null) {
+	if (names == null) {
 		this.exceptionNames = noException;
+	} else {
+		this.exceptionNames = names;
 	}
 }
-private void readModifierRelatedAttributes() {
+private synchronized void readModifierRelatedAttributes() {
+	int flags = u2At(0);
 	int attributesCount = u2At(6);
 	int readOffset = 8;
 	for (int i = 0; i < attributesCount; i++) {
@@ -421,23 +420,24 @@ private void readModifierRelatedAttributes() {
 			switch(attributeName[0]) {
 				case 'D' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.DeprecatedName))
-						this.accessFlags |= ClassFileConstants.AccDeprecated;
+						flags |= ClassFileConstants.AccDeprecated;
 					break;
 				case 'S' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.SyntheticName))
-						this.accessFlags |= ClassFileConstants.AccSynthetic;
+						flags |= ClassFileConstants.AccSynthetic;
 					break;
 				case 'A' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.AnnotationDefaultName))
-						this.accessFlags |= ClassFileConstants.AccAnnotationDefault;
+						flags |= ClassFileConstants.AccAnnotationDefault;
 					break;
 				case 'V' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.VarargsName))
-						this.accessFlags |= ClassFileConstants.AccVarargs;
+						flags |= ClassFileConstants.AccVarargs;
 			}
 		}
 		readOffset += (6 + u4At(readOffset + 2));
 	}
+	this.accessFlags = flags;
 }
 /**
  * Answer the size of the receiver in bytes.
@@ -461,7 +461,7 @@ void toString(StringBuffer buffer) {
 protected void toStringContent(StringBuffer buffer) {
 	BinaryTypeFormatter.methodToStringContent(buffer, this);
 }
-private void readCodeAttribute() {
+private synchronized void readCodeAttribute() {
 	int attributesCount = u2At(6);
 	int readOffset = 8;
 	if (attributesCount != 0) {
@@ -508,7 +508,7 @@ private void decodeLocalVariableAttribute(int offset, int codeLength) {
 	final int length = u2At(readOffset);
 	if (length != 0) {
 		readOffset += 2;
-		this.argumentNames = new char[length][];
+		char[][] names = new char[length][];
 		int argumentNamesIndex = 0;
 		for (int i = 0; i < length; i++) {
 			int startPC = u2At(readOffset);
@@ -517,17 +517,18 @@ private void decodeLocalVariableAttribute(int offset, int codeLength) {
 				int utf8Offset = this.constantPoolOffsets[nameIndex] - this.structOffset;
 				char[] localVariableName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
 				if (!CharOperation.equals(localVariableName, ConstantPool.This)) {
-					this.argumentNames[argumentNamesIndex++] = localVariableName;
+					names[argumentNamesIndex++] = localVariableName;
 				}
 			} else {
 				break;
 			}
 			readOffset += 10;
 		}
-		if (argumentNamesIndex != this.argumentNames.length) {
+		if (argumentNamesIndex != names.length) {
 			// resize
-			System.arraycopy(this.argumentNames, 0, (this.argumentNames = new char[argumentNamesIndex][]), 0, argumentNamesIndex);
+			System.arraycopy(names, 0, (names = new char[argumentNamesIndex][]), 0, argumentNamesIndex);
 		}
+		this.argumentNames = names;
 	}
 }
 private void decodeMethodParameters(int offset, MethodInfo methodInfo) {
@@ -535,18 +536,19 @@ private void decodeMethodParameters(int offset, MethodInfo methodInfo) {
 	final int length = u1At(readOffset);
 	if (length != 0) {
 		readOffset += 1;
-		this.argumentNames = new char[length][];
+		char[][] names = new char[length][];
 		for (int i = 0; i < length; i++) {
 			int nameIndex = u2At(readOffset);
 			if (nameIndex != 0) {
 				int utf8Offset = this.constantPoolOffsets[nameIndex] - this.structOffset;
 				char[] parameterName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
-				this.argumentNames[i] = parameterName;
+				names[i] = parameterName;
 			} else {
-				this.argumentNames[i] = CharOperation.concat(ARG, String.valueOf(i).toCharArray());
+				names[i] = CharOperation.concat(ARG, String.valueOf(i).toCharArray());
 			}
 			readOffset += 4;
 		}
+		this.argumentNames = names;
 	}
 }
 }
