@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -892,7 +895,7 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 
 		return index;
 	}
-	public int literalIndexForInvokeDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
+	private int literalIndexForInvokeAndConstantDynamic(int bootStrapIndex, char[] selector, char[] descriptor, int tag) {
 		int index;
 		if ((index = putInDynamicCacheIfAbsent(bootStrapIndex, selector, descriptor, this.currentIndex)) < 0) {
 			this.currentIndex++;
@@ -906,7 +909,7 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 			}
 			this.offsets[index] = this.currentOffset;
 
-			writeU1(InvokeDynamicTag);
+			writeU1(tag);
 			int classIndexOffset = this.currentOffset;
 			if (this.currentOffset + 4 >= this.poolContent.length) {
 				resizePoolContents(4);
@@ -922,6 +925,13 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 		}
 		return index;
 	}
+	// CONSTANT_Dynamic_info JVMS 4.4.10 /jep 309
+	public int literalIndexForDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
+		return literalIndexForInvokeAndConstantDynamic(bootStrapIndex, selector, descriptor, DynamicTag);
+	}
+	public int literalIndexForInvokeDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
+		return literalIndexForInvokeAndConstantDynamic(bootStrapIndex, selector, descriptor, InvokeDynamicTag);
+	}
 	public int literalIndexForField(char[] declaringClass, char[] name, char[] signature) {
 		int index;
 		if ((index = putInCacheIfAbsent(declaringClass, name, signature, this.currentIndex)) < 0) {
