@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -1003,6 +1006,7 @@ public class Util implements SuffixConstants {
 	public static String toString(Object[] objects) {
 		return toString(objects,
 			new Displayable(){
+				@Override
 				public String displayString(Object o) {
 					if (o == null) return "null"; //$NON-NLS-1$
 					return o.toString();
@@ -1241,7 +1245,7 @@ public class Util implements SuffixConstants {
 		}
 		List<FileSystem.Classpath> classpaths = new ArrayList<>();
 		for (String filePath : filePaths) {
-			FileSystem.Classpath currentClasspath = FileSystem.getClasspath(filePath, null, null, null);
+			FileSystem.Classpath currentClasspath = FileSystem.getClasspath(filePath, null, null, null, null);
 			if (currentClasspath != null) {
 				classpaths.add(currentClasspath);
 			}
