@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -1358,25 +1361,19 @@ public abstract class ASTNode {
 			 */
 			private int position = 0;
 
-			/* (non-Javadoc)
-			 * Method declared on <code>Iterator</code>.
-			 */
+			@Override
 			public boolean hasNext() {
 				return this.position < NodeList.this.store.size();
 			}
 
-			/* (non-Javadoc)
-			 * Method declared on <code>Iterator</code>.
-			 */
+			@Override
 			public Object next() {
 				Object result = NodeList.this.store.get(this.position);
 				this.position++;
 				return result;
 		    }
 
-			/* (non-Javadoc)
-			 * Method declared on <code>Iterator</code>.
-			 */
+			@Override
 			public void remove() {
 				throw new UnsupportedOperationException();
 			}
@@ -1425,23 +1422,17 @@ public abstract class ASTNode {
 			this.propertyDescriptor = property;
 		}
 
-		/* (non-javadoc)
-		 * @see java.util.AbstractCollection#size()
-		 */
+		@Override
 		public int size() {
 			return this.store.size();
 		}
 
-		/* (non-javadoc)
-		 * @see AbstractList#get(int)
-		 */
+		@Override
 		public Object get(int index) {
 			return this.store.get(index);
 		}
 
-		/* (non-javadoc)
-		 * @see List#set(int, java.lang.Object)
-		 */
+		@Override
 		public Object set(int index, Object element) {
 		    if (element == null) {
 		        throw new IllegalArgumentException();
@@ -1471,9 +1462,7 @@ public abstract class ASTNode {
 			return result;
 		}
 
-		/* (non-javadoc)
-		 * @see List#add(int, java.lang.Object)
-		 */
+		@Override
 		public void add(int index, Object element) {
 		    if (element == null) {
 		        throw new IllegalArgumentException();
@@ -1495,9 +1484,7 @@ public abstract class ASTNode {
 			ASTNode.this.ast.postAddChildEvent(ASTNode.this, newChild, this.propertyDescriptor);
 		}
 
-		/* (non-javadoc)
-		 * @see List#remove(int)
-		 */
+		@Override
 		public Object remove(int index) {
 			if ((ASTNode.this.typeAndFlags & PROTECT) != 0) {
 				// this node is protected => cannot gain or lose children
@@ -2055,6 +2042,36 @@ public abstract class ASTNode {
 		}
 	}
 	/**
+     * Checks that this AST operation is not used when
+     * building JLS2, JLS3, JLS4, JLS8 or JLS9 level ASTs.
+     * <p>
+     * Use this method to prevent access to new properties that have been added in JLS10
+     * </p>
+     *
+	 * @exception UnsupportedOperationException if this operation is used below JLS10
+	 * @since 3.14
+	 */
+	final void unsupportedBelow10() {
+		if (this.ast.apiLevel < AST.JLS10_INTERNAL) {
+			throw new UnsupportedOperationException("Operation only supported in ASTs with level JLS10 and above"); //$NON-NLS-1$
+		}
+	}
+	/**
+     * Checks that this AST operation is not used when
+     * building JLS2, JLS3, JLS4, JLS8, JLS9 or JLS10 level ASTs.
+     * <p>
+     * Use this method to prevent access to new properties that have been added in JLS11
+     * </p>
+     *
+	 * @exception UnsupportedOperationException if this operation is used below JLS11
+	 * @since 3.14 
+	 */
+	final void unsupportedBelow11() {
+		if (this.ast.apiLevel < AST.JLS11_INTERNAL) {
+			throw new UnsupportedOperationException("Operation only supported in ASTs with level JLS11 and above"); //$NON-NLS-1$
+		}
+	}
+	/**
      * Checks that this AST operation is only used when
      * building JLS2 level ASTs.
      * <p>
@@ -2178,7 +2195,7 @@ public abstract class ASTNode {
 		Class childClass = newChild.getClass();
 		if (nodeType != null && !nodeType.isAssignableFrom(childClass)) {
 			// new child is not of the right type
-			throw new ClassCastException();
+			throw new ClassCastException(childClass + " is not an instance of " + nodeType); //$NON-NLS-1$
 		}
 		if ((newChild.typeAndFlags & PROTECT) != 0) {
 			// new child node is protected => cannot be parented
@@ -2620,6 +2637,7 @@ public abstract class ASTNode {
 	 * @return {@inheritDoc}
 	 * @see #subtreeMatch(ASTMatcher matcher, Object other)
 	 */
+	@Override
 	public final boolean equals(Object obj) {
 		return this == obj; // equivalent to Object.equals
 	}
@@ -2629,6 +2647,7 @@ public abstract class ASTNode {
 	 * This makes it consistent with the fact that a equals methods has been provided.
 	 * @see java.lang.Object#hashCode()
 	 */
+	@Override
 	public final int hashCode() {
 		return super.hashCode();
 	}
@@ -2945,6 +2964,7 @@ public abstract class ASTNode {
 	 *
 	 * @return a debug string
 	 */
+	@Override
 	public final String toString() {
 		StringBuffer buffer = new StringBuffer();
 		int p = buffer.length();
