@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2004, 2018 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -230,6 +233,7 @@ public class ASTParser {
 			case AST.JLS8_INTERNAL:
 			case AST.JLS9_INTERNAL:
 			case AST.JLS10_INTERNAL:
+			case AST.JLS11_INTERNAL:
 				break;
 			default:
 				throw new IllegalArgumentException();
