@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2004, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -91,6 +94,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.docComment.setComment(new String(this.source, start, length));
 	}
 
+	@Override
 	public String toString() {
 		StringBuffer buffer = new StringBuffer();
 		buffer.append("javadoc: ").append(this.docComment).append("\n");	//$NON-NLS-1$ //$NON-NLS-2$
@@ -98,9 +102,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return buffer.toString();
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createArgumentReference(char[], java.lang.Object, int)
-	 */
+	@Override
 	protected Object createArgumentReference(char[] name, int dim, boolean isVarargs, Object typeRef, long[] dimPositions, long argNamePos) throws InvalidInputException {
 		try {
 			MethodRefParameter argument = this.ast.newMethodRefParameter();
@@ -154,9 +156,8 @@ class DocCommentParser extends AbstractCommentParser {
 				throw new InvalidInputException();
 		}
 	}
-/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createFieldReference()
-	 */
+
+	@Override
 	protected Object createFieldReference(Object receiver) throws InvalidInputException {
 		try {
 			MemberRef fieldRef = this.ast.newMemberRef();
@@ -182,9 +183,8 @@ class DocCommentParser extends AbstractCommentParser {
 				throw new InvalidInputException();
 		}
 	}
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createMethodReference(java.lang.Object[])
-	 */
+
+	@Override
 	protected Object createMethodReference(Object receiver, List arguments) throws InvalidInputException {
 		try {
 			// Create method ref
@@ -221,9 +221,7 @@ class DocCommentParser extends AbstractCommentParser {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createTag()
-	 */
+	@Override
 	protected void createTag() {
 		TagElement tagElement = this.ast.newTagElement();
 		int position = this.scanner.currentPosition;
@@ -256,9 +254,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.scanner.resetTo(position, this.javadocEnd);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createTypeReference()
-	 */
+	@Override
 	protected Object createTypeReference(int primitiveToken) {
 		int size = this.identifierLengthStack[this.identifierLengthPtr];
 		String[] identifiers = new String[size];
@@ -331,9 +327,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return typeRef;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseIdentifierTag(boolean)
-	 */
+	@Override
 	protected boolean parseIdentifierTag(boolean report) {
 		if (super.parseIdentifierTag(report)) {
 			createTag();
@@ -352,9 +346,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseTag(int)
-	 */
+	@Override
 	protected boolean parseTag(int previousPosition) throws InvalidInputException {
 
 		// Read tag name
@@ -595,9 +587,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return valid;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushParamName(java.lang.Object)
-	 */
+	@Override
 	protected boolean pushParamName(boolean isTypeParam) {
 		int idIndex = isTypeParam ? 1 : 0;
 		final SimpleName name = new SimpleName(this.ast);
@@ -633,9 +623,8 @@ class DocCommentParser extends AbstractCommentParser {
 		pushOnAstStack(paramTag, true);
 		return true;
 	}
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushSeeRef(java.lang.Object)
-	 */
+
+	@Override
 	protected boolean pushSeeRef(Object statement) {
 		TagElement seeTag = this.ast.newTagElement();
 		ASTNode node = (ASTNode) statement;
@@ -672,9 +661,8 @@ class DocCommentParser extends AbstractCommentParser {
 		}
 		return true;
 	}
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushText(int, int)
-	 */
+
+	@Override
 	protected void pushText(int start, int end) {
 
 		// Create text element
@@ -719,9 +707,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.textStart = -1;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushThrowName(java.lang.Object)
-	 */
+	@Override
 	protected boolean pushThrowName(Object typeRef) {
 		TagElement throwsTag = this.ast.newTagElement();
 		switch (this.tagValue) {
@@ -738,9 +724,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#refreshInlineTagPosition(int)
-	 */
+	@Override
 	protected void refreshInlineTagPosition(int previousPosition) {
 		if (this.astPtr != -1) {
 			TagElement previousTag = (TagElement) this.astStack[this.astPtr];
@@ -761,6 +745,7 @@ class DocCommentParser extends AbstractCommentParser {
 	/*
 	 * Add stored tag elements to associated comment.
 	 */
+	@Override
 	protected void updateDocComment() {
 		for (int idx = 0; idx <= this.astPtr; idx++) {
 			this.docComment.tags().add(this.astStack[idx]);
