@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2016 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2018 Mateusz Matela and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
@@ -14,6 +17,7 @@ package org.eclipse.jdt.internal.formatter;
 
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.*;
 
+import java.util.ArrayList;
 import java.util.List;
 
 import org.eclipse.jdt.core.dom.ASTNode;
@@ -37,6 +41,7 @@ import org.eclipse.jdt.core.dom.ConstructorInvocation;
 import org.eclipse.jdt.core.dom.CreationReference;
 import org.eclipse.jdt.core.dom.Dimension;
 import org.eclipse.jdt.core.dom.DoStatement;
+import org.eclipse.jdt.core.dom.EmptyStatement;
 import org.eclipse.jdt.core.dom.EnhancedForStatement;
 import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
 import org.eclipse.jdt.core.dom.EnumDeclaration;
@@ -226,13 +231,21 @@ public class SpacePreparator extends ASTVisitor {
 				: this.options.insert_space_before_opening_brace_in_method_declaration) && node.getBody() != null)
 			this.tm.firstTokenIn(node.getBody(), TokenNameLBRACE).spaceBefore();
 
+		if (node.getReceiverType() != null)
+			this.tm.lastTokenIn(node.getReceiverType(), -1).spaceAfter();
+
 		boolean beforeComma = node.isConstructor()
 				? this.options.insert_space_before_comma_in_constructor_declaration_parameters
 				: this.options.insert_space_before_comma_in_method_declaration_parameters;
 		boolean afterComma = node.isConstructor()
 				? this.options.insert_space_after_comma_in_constructor_declaration_parameters
 				: this.options.insert_space_after_comma_in_method_declaration_parameters;
-		handleCommas(node.parameters(), beforeComma, afterComma);
+		List<SingleVariableDeclaration> params = node.parameters();
+		if (node.getReceiverType() != null) {
+			params = new ArrayList<>(params);
+			params.add(0, null); // space for explicit receiver, null OK - first value not read in handleCommas 
+		}
+		handleCommas(params, beforeComma, afterComma);
 
 		List<Type> thrownExceptionTypes = node.thrownExceptionTypes();
 		if (!thrownExceptionTypes.isEmpty()) {
@@ -354,6 +367,7 @@ public class SpacePreparator extends ASTVisitor {
 				this.options.insert_space_after_opening_paren_in_while);
 		handleTokenBefore(node.getBody(), TokenNameRPAREN, this.options.insert_space_before_closing_paren_in_while,
 				false);
+		handleLoopBody(node.getBody());
 		return true;
 	}
 
@@ -555,16 +569,9 @@ public class SpacePreparator extends ASTVisitor {
 				this.options.insert_space_after_opening_paren_in_if);
 
 		Statement thenStatement = node.getThenStatement();
-		int closingParenIndex = this.tm.firstIndexBefore(thenStatement, TokenNameRPAREN);
-		handleToken(this.tm.get(closingParenIndex), this.options.insert_space_before_closing_paren_in_if,
-				/* space before then statement may be needed if it will stay on the same line */
-				!(thenStatement instanceof Block) && !this.tm.get(closingParenIndex + 1).isComment());
-
-		if (thenStatement instanceof Block && this.tm.isGuardClause((Block) thenStatement)) {
-			handleToken(thenStatement, TokenNameLBRACE, false, true);
-			this.tm.lastTokenIn(node, TokenNameRBRACE).spaceBefore();
-		}
+		handleTokenBefore(thenStatement, TokenNameRPAREN, this.options.insert_space_before_closing_paren_in_if, false);
 
+		handleLoopBody(thenStatement);
 		handleSemicolon(thenStatement);
 		return true;
 	}
@@ -588,6 +595,8 @@ public class SpacePreparator extends ASTVisitor {
 		handleTokenBefore(node.getBody(), TokenNameSEMICOLON,
 				this.options.insert_space_before_semicolon_in_for && !part2Empty,
 				this.options.insert_space_after_semicolon_in_for && !part3Empty);
+
+		handleLoopBody(node.getBody());
 		return true;
 	}
 
@@ -613,6 +622,7 @@ public class SpacePreparator extends ASTVisitor {
 				this.options.insert_space_before_closing_paren_in_for, false);
 		handleTokenAfter(node.getParameter(), TokenNameCOLON, this.options.insert_space_before_colon_in_for,
 				this.options.insert_space_after_colon_in_for);
+		handleLoopBody(node.getBody());
 		return true;
 	}
 
@@ -1073,6 +1083,15 @@ public class SpacePreparator extends ASTVisitor {
 		}
 	}
 
+	private void handleLoopBody(Statement loopBody) {
+		/* space before body statement may be needed if it will stay on the same line */
+		int firstTokenIndex = this.tm.firstIndexIn(loopBody, -1);
+		if (!(loopBody instanceof Block) && !(loopBody instanceof EmptyStatement)
+				&& !this.tm.get(firstTokenIndex - 1).isComment()) {
+			this.tm.get(firstTokenIndex).spaceBefore();
+		}
+	}
+
 	public void finishUp() {
 		this.tm.traverse(0, new TokenTraverser() {
 			boolean isPreviousJIDP = false;
