@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,7 +16,8 @@ package org.eclipse.jdt.internal.core.builder;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
-
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
+import org.eclipse.core.runtime.preferences.InstanceScope;
 import org.eclipse.jdt.core.*;
 import org.eclipse.jdt.core.compiler.*;
 import org.eclipse.jdt.internal.compiler.util.SimpleLookupTable;
@@ -27,11 +31,13 @@ import java.util.*;
 @SuppressWarnings({"rawtypes", "unchecked"})
 public class JavaBuilder extends IncrementalProjectBuilder {
 
+public static final String PREF_NULL_SCHEDULING_RULE = "useNullSchedulingRule"; //$NON-NLS-1$
 IProject currentProject;
 JavaProject javaProject;
 IWorkspaceRoot workspaceRoot;
 CompilationParticipant[] participants;
 NameEnvironment nameEnvironment;
+NameEnvironment testNameEnvironment;
 SimpleLookupTable binaryLocationsPerProject; // maps a project to its binary resources (output folders, class folders, zip/jar files)
 public State lastState;
 BuildNotifier notifier;
@@ -47,7 +53,7 @@ public static boolean SHOW_STATS = false;
  * This list is used to reset the JavaModel.existingExternalFiles cache when a build cycle begins
  * so that deleted external jars are discovered.
  */
-static ArrayList builtProjects = null;
+static LinkedHashSet<String> builtProjects;
 
 public static IMarker[] getProblemsFor(IResource resource) {
 	try {
@@ -154,6 +160,7 @@ public static void writeState(Object state, DataOutputStream out) throws IOExcep
 	((State) state).write(out);
 }
 
+@Override
 protected IProject[] build(int kind, Map ignored, IProgressMonitor monitor) throws CoreException {
 	this.currentProject = getProject();
 	if (this.currentProject == null || !this.currentProject.isAccessible()) return new IProject[0];
@@ -184,7 +191,7 @@ protected IProject[] build(int kind, Map ignored, IProgressMonitor monitor) thro
 					if (DEBUG)
 						System.out.println("JavaBuilder: Performing full build since classpath has changed"); //$NON-NLS-1$
 					buildAll();
-				} else if (this.nameEnvironment.sourceLocations.length > 0) {
+				} else if (this.nameEnvironment.sourceLocations.length > 0 || this.testNameEnvironment.sourceLocations.length > 0) {
 					// if there is no source to compile & no classpath changes then we are done
 					SimpleLookupTable deltas = findDeltas();
 					if (deltas == null) {
@@ -252,8 +259,15 @@ private void buildAll() {
 	if (DEBUG && this.lastState != null)
 		System.out.println("JavaBuilder: Clearing last state : " + this.lastState); //$NON-NLS-1$
 	clearLastState();
-	BatchImageBuilder imageBuilder = new BatchImageBuilder(this, true);
+	BatchImageBuilder imageBuilder = new BatchImageBuilder(this, true, CompilationGroup.MAIN);
+	BatchImageBuilder testImageBuilder = new BatchImageBuilder(imageBuilder, true, CompilationGroup.TEST);
 	imageBuilder.build();
+	if (testImageBuilder.sourceLocations.length > 0) {
+		// Note: testImageBuilder *MUST* have a separate output folder, or it will delete the files created by imageBuilder.build() 
+		testImageBuilder.build();
+	} else {
+		testImageBuilder.cleanUp();
+	}
 	recordNewState(imageBuilder.newState);
 }
 
@@ -273,6 +287,7 @@ private void buildDeltas(SimpleLookupTable deltas) {
 	}
 }
 
+@Override
 protected void clean(IProgressMonitor monitor) throws CoreException {
 	this.currentProject = getProject();
 	if (this.currentProject == null || !this.currentProject.isAccessible()) return;
@@ -290,7 +305,8 @@ protected void clean(IProgressMonitor monitor) throws CoreException {
 			System.out.println("JavaBuilder: Clearing last state as part of clean : " + this.lastState); //$NON-NLS-1$
 		clearLastState();
 		removeProblemsAndTasksFor(this.currentProject);
-		new BatchImageBuilder(this, false).cleanOutputFolders(false);
+		new BatchImageBuilder(this, false, CompilationGroup.MAIN).cleanOutputFolders(false);
+		new BatchImageBuilder(this, false, CompilationGroup.TEST).cleanOutputFolders(false);
 	} catch (CoreException e) {
 		Util.log(e, "JavaBuilder handling CoreException while cleaning: " + this.currentProject.getName()); //$NON-NLS-1$
 		createInconsistentBuildMarker(e);
@@ -329,6 +345,7 @@ private void createInconsistentBuildMarker(CoreException coreException) throws C
 private void cleanup() {
 	this.participants = null;
 	this.nameEnvironment = null;
+	this.testNameEnvironment = null;
 	this.binaryLocationsPerProject = null;
 	this.lastState = null;
 	this.notifier = null;
@@ -431,7 +448,7 @@ public State getLastState(IProject project) {
 private IProject[] getRequiredProjects(boolean includeBinaryPrerequisites) {
 	if (this.javaProject == null || this.workspaceRoot == null) return new IProject[0];
 
-	ArrayList projects = new ArrayList();
+	LinkedHashSet<IProject> projects = new LinkedHashSet<>();
 	ExternalFoldersManager externalFoldersManager = JavaModelManager.getExternalManager();
 	try {
 		IClasspathEntry[] entries = this.javaProject.getExpandedClasspath();
@@ -478,8 +495,12 @@ boolean hasBuildpathErrors() throws CoreException {
 }
 
 private boolean hasClasspathChanged() {
-	ClasspathMultiDirectory[] newSourceLocations = this.nameEnvironment.sourceLocations;
-	ClasspathMultiDirectory[] oldSourceLocations = this.lastState.sourceLocations;
+	return hasClasspathChanged(CompilationGroup.MAIN) || hasClasspathChanged(CompilationGroup.TEST);	
+}
+
+private boolean hasClasspathChanged(CompilationGroup compilationGroup) {
+	ClasspathMultiDirectory[] newSourceLocations = (compilationGroup == CompilationGroup.MAIN ? this.nameEnvironment : this.testNameEnvironment).sourceLocations;
+	ClasspathMultiDirectory[] oldSourceLocations = compilationGroup == CompilationGroup.MAIN ? this.lastState.sourceLocations : this.lastState.testSourceLocations;
 	int newLength = newSourceLocations.length;
 	int oldLength = oldSourceLocations.length;
 	int n, o;
@@ -527,8 +548,8 @@ private boolean hasClasspathChanged() {
 		return true;
 	}
 
-	ClasspathLocation[] newBinaryLocations = this.nameEnvironment.binaryLocations;
-	ClasspathLocation[] oldBinaryLocations = this.lastState.binaryLocations;
+	ClasspathLocation[] newBinaryLocations = (compilationGroup == CompilationGroup.MAIN ? this.nameEnvironment : this.testNameEnvironment).binaryLocations;
+	ClasspathLocation[] oldBinaryLocations = compilationGroup == CompilationGroup.MAIN ? this.lastState.binaryLocations : this.lastState.testBinaryLocations;
 	newLength = newBinaryLocations.length;
 	oldLength = oldBinaryLocations.length;
 	for (n = o = 0; n < newLength && o < oldLength; n++, o++) {
@@ -595,13 +616,14 @@ private int initializeBuilder(int kind, boolean forBuild) throws CoreException {
 		// Flush the existing external files cache if this is the beginning of a build cycle
 		String projectName = this.currentProject.getName();
 		if (builtProjects == null || builtProjects.contains(projectName)) {
-			builtProjects = new ArrayList();
+			builtProjects = new LinkedHashSet();
 		}
 		builtProjects.add(projectName);
 	}
 
 	this.binaryLocationsPerProject = new SimpleLookupTable(3);
-	this.nameEnvironment = new NameEnvironment(this.workspaceRoot, this.javaProject, this.binaryLocationsPerProject, this.notifier);
+	this.nameEnvironment = new NameEnvironment(this.workspaceRoot, this.javaProject, this.binaryLocationsPerProject, this.notifier, CompilationGroup.MAIN);
+	this.testNameEnvironment = new NameEnvironment(this.workspaceRoot, this.javaProject, this.binaryLocationsPerProject, this.notifier, CompilationGroup.TEST);
 
 	if (forBuild) {
 		String filterSequence = this.javaProject.getOption(JavaCore.CORE_JAVA_BUILD_RESOURCE_COPY_FILTER, true);
@@ -775,9 +797,18 @@ private void recordNewState(State state) {
 /**
  * String representation for debugging purposes
  */
+@Override
 public String toString() {
 	return this.currentProject == null
 		? "JavaBuilder for unknown project" //$NON-NLS-1$
 		: "JavaBuilder for " + this.currentProject.getName(); //$NON-NLS-1$
 }
+
+	@Override
+	public ISchedulingRule getRule(int kind, Map<String, String> args) {
+		if (InstanceScope.INSTANCE.getNode(JavaCore.PLUGIN_ID).getBoolean(PREF_NULL_SCHEDULING_RULE, false)) {
+			return null;
+		}
+		return super.getRule(kind, args);
+	}
 }
