@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -48,18 +51,21 @@ BuildNotifier notifier;
 
 SimpleSet initialTypeNames; // assumed that each name is of the form "a/b/ClassName", or, if a module is given: "my.mod:a/b/ClassName"
 SimpleLookupTable additionalUnits;
+private CompilationGroup compilationGroup;
 /** Tasks resulting from add-reads or add-exports classpath attributes. */
 ModuleUpdater moduleUpdater;
 
-NameEnvironment(IWorkspaceRoot root, JavaProject javaProject, SimpleLookupTable binaryLocationsPerProject, BuildNotifier notifier) throws CoreException {
+NameEnvironment(IWorkspaceRoot root, JavaProject javaProject, SimpleLookupTable binaryLocationsPerProject, BuildNotifier notifier, CompilationGroup compilationGroup) throws CoreException {
+	this.compilationGroup = compilationGroup;
 	this.isIncrementalBuild = false;
 	this.notifier = notifier;
 	computeClasspathLocations(root, javaProject, binaryLocationsPerProject);
 	setNames(null, null);
 }
 
-public NameEnvironment(IJavaProject javaProject) {
+public NameEnvironment(IJavaProject javaProject, CompilationGroup compilationGroup) {
 	this.isIncrementalBuild = false;
+	this.compilationGroup = compilationGroup;
 	try {
 		computeClasspathLocations(javaProject.getProject().getWorkspace().getRoot(), (JavaProject) javaProject, null);
 	} catch(CoreException e) {
@@ -106,13 +112,18 @@ private void computeClasspathLocations(
 			cycleMarker.setAttribute(IMarker.SEVERITY, severity);
 	}
 
-	IClasspathEntry[] classpathEntries = javaProject.getExpandedClasspath();
+	IClasspathEntry[] classpathEntries = javaProject.getExpandedClasspath(this.compilationGroup == CompilationGroup.MAIN);
 	ArrayList sLocations = new ArrayList(classpathEntries.length);
 	ArrayList bLocations = new ArrayList(classpathEntries.length);
+	ArrayList sLocationsForTest = new ArrayList(classpathEntries.length);
 	Map<String, IModulePathEntry> moduleEntries = null;
-	if (CompilerOptions.versionToJdkLevel(javaProject.getOption(JavaCore.COMPILER_COMPLIANCE, true)) >= ClassFileConstants.JDK9) {
-		moduleEntries = new HashMap<>(classpathEntries.length);
+	String compliance = javaProject.getOption(JavaCore.COMPILER_COMPLIANCE, true);
+	if (CompilerOptions.versionToJdkLevel(compliance) >= ClassFileConstants.JDK9) {
+		moduleEntries = new LinkedHashMap<>(classpathEntries.length);
 		this.moduleUpdater = new ModuleUpdater(javaProject);
+		if (this.compilationGroup == CompilationGroup.TEST) {
+			this.moduleUpdater.addReadUnnamedForNonEmptyClasspath(javaProject, classpathEntries);
+		}
 	}
 	IModuleDescription projectModule = javaProject.getModuleDescription();
 
@@ -139,7 +150,7 @@ private void computeClasspathLocations(
 			patchedModuleName = null;
 		}
 
-		if (this.moduleUpdater != null)
+		if (this.moduleUpdater != null && (this.compilationGroup == CompilationGroup.TEST || !entry.isTest()))
 			this.moduleUpdater.computeModuleUpdates(entry);
 
 		switch(entry.getEntryKind()) {
@@ -156,17 +167,27 @@ private void computeClasspathLocations(
 					if (!outputFolder.exists())
 						createOutputFolder(outputFolder);
 				}
-				ClasspathLocation sourceLocation = ClasspathLocation.forSourceFolder(
-							(IContainer) target, 
-							outputFolder,
-							entry.fullInclusionPatternChars(), 
-							entry.fullExclusionPatternChars(),
-							entry.ignoreOptionalProblems());
-				if (patchedModule != null) {
-					ModuleEntryProcessor.combinePatchIntoModuleEntry(sourceLocation, patchedModule, moduleEntries);
+				if (this.compilationGroup == CompilationGroup.TEST && !entry.isTest()) {
+					ClasspathLocation bLocation = ClasspathLocation.forBinaryFolder(outputFolder, true, entry.getAccessRuleSet(), externalAnnotationPath, isOnModulePath);
+					bLocations.add(bLocation);
+					sLocationsForTest.add(bLocation);
+					if (patchedModule != null) {
+						ModuleEntryProcessor.combinePatchIntoModuleEntry(bLocation, patchedModule, moduleEntries);
+					}
+					bLocation.patchModuleName = patchedModuleName;
+				} else {
+					ClasspathLocation sourceLocation = ClasspathLocation.forSourceFolder(
+								(IContainer) target, 
+								outputFolder,
+								entry.fullInclusionPatternChars(), 
+								entry.fullExclusionPatternChars(),
+								entry.ignoreOptionalProblems());
+					if (patchedModule != null) {
+						ModuleEntryProcessor.combinePatchIntoModuleEntry(sourceLocation, patchedModule, moduleEntries);
+					}
+					sLocations.add(sourceLocation);
 					sourceLocation.patchModuleName = patchedModuleName;
 				}
-				sLocations.add(sourceLocation);
 				continue nextEntry;
 
 			case IClasspathEntry.CPE_PROJECT :
@@ -181,6 +202,8 @@ private void computeClasspathLocations(
 				nextPrereqEntry: for (int j = 0, m = prereqClasspathEntries.length; j < m; j++) {
 					IClasspathEntry prereqEntry = prereqClasspathEntries[j];
 					if (prereqEntry.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
+						if ((this.compilationGroup == CompilationGroup.MAIN || entry.isWithoutTestCode()) && prereqEntry.isTest())
+							continue nextPrereqEntry;
 						Object prereqTarget = JavaModel.getTarget(prereqEntry.getPath(), true);
 						if (!(prereqTarget instanceof IContainer)) continue nextPrereqEntry;
 						IPath prereqOutputPath = prereqEntry.getOutputLocation() != null
@@ -213,8 +236,8 @@ private void computeClasspathLocations(
 					try {
 						IModuleDescription mod;
 						if ((mod = prereqJavaProject.getModuleDescription()) != null) {
-							SourceModule sourceModule = (SourceModule) mod;
-							info = (ModuleDescriptionInfo) sourceModule.getElementInfo();
+							AbstractModule aModule = (AbstractModule) mod;
+							info = aModule.getModuleInfo();
 						}
 					} catch (JavaModelException jme) {
 						// do nothing, probably a non module project
@@ -248,7 +271,7 @@ private void computeClasspathLocations(
 							&& JavaCore.IGNORE.equals(javaProject.getOption(JavaCore.COMPILER_PB_DISCOURAGED_REFERENCE, true)))
 								? null
 								: entry.getAccessRuleSet();
-						bLocation = ClasspathLocation.forLibrary((IFile) resource, accessRuleSet, externalAnnotationPath, isOnModulePath);
+						bLocation = ClasspathLocation.forLibrary((IFile) resource, accessRuleSet, externalAnnotationPath, isOnModulePath, compliance);
 					} else if (resource instanceof IContainer) {
 						AccessRuleSet accessRuleSet =
 							(JavaCore.IGNORE.equals(javaProject.getOption(JavaCore.COMPILER_PB_FORBIDDEN_REFERENCE, true))
@@ -282,7 +305,14 @@ private void computeClasspathLocations(
 							&& JavaCore.IGNORE.equals(javaProject.getOption(JavaCore.COMPILER_PB_DISCOURAGED_REFERENCE, true)))
 								? null
 								: entry.getAccessRuleSet();
-					ClasspathLocation bLocation = ClasspathLocation.forLibrary(path.toOSString(), accessRuleSet, externalAnnotationPath, isOnModulePath);
+					String release = JavaCore.DISABLED.equals(javaProject.getOption(JavaCore.COMPILER_RELEASE, true)) ? null : compliance;
+					ClasspathLocation bLocation = null;
+					String libPath = path.toOSString();
+					if (Util.isJrt(libPath)) {
+						bLocation = ClasspathLocation.forJrtSystem(path.toOSString(), accessRuleSet, externalAnnotationPath, release);
+					} else {
+						bLocation = ClasspathLocation.forLibrary(path.toOSString(), accessRuleSet, externalAnnotationPath, isOnModulePath, compliance);
+					}
 					bLocations.add(bLocation);
 					if (moduleEntries != null) {
 						Set<String> libraryLimitModules = (limitModules == null && projectModule != null) ? ClasspathJrt.NO_LIMIT_MODULES : limitModules;
@@ -302,8 +332,18 @@ private void computeClasspathLocations(
 		if (moduleEntries != null && projectModule != null) {
 			try {
 				AbstractModule sourceModule = (AbstractModule)projectModule;
-				ModuleDescriptionInfo info = (ModuleDescriptionInfo) sourceModule.getElementInfo();
-				ModulePathEntry projectEntry = new ModulePathEntry(javaProject.getPath(), info, this.sourceLocations);
+				IModule info = (IModule) sourceModule.getElementInfo();
+				final ClasspathLocation[] sourceLocations2;
+				if(sLocationsForTest.size() == 0) {
+					sourceLocations2 = this.sourceLocations;
+				} else {
+					ArrayList<ClasspathLocation> sourceLocationsForModulePathEntry=new ArrayList<>(sLocations.size()+sLocationsForTest.size());
+					sourceLocationsForModulePathEntry.addAll(sLocations);
+					sourceLocationsForModulePathEntry.addAll(sLocationsForTest);
+					sourceLocations2= sourceLocationsForModulePathEntry
+							.toArray(new ClasspathLocation[sourceLocationsForModulePathEntry.size()]);
+				}
+				ModulePathEntry projectEntry = new ModulePathEntry(javaProject.getPath(), info, sourceLocations2);
 				if (!moduleEntries.containsKey(sourceModule.getElementName())) { // can be registered already, if patching
 					moduleEntries.put(sourceModule.getElementName(), projectEntry);
 				}
@@ -402,6 +442,7 @@ protected boolean isOnModulePath(ClasspathEntry entry) {
 	return entry.isModular();
 }
 
+@Override
 public void cleanup() {
 	this.initialTypeNames = null;
 	this.additionalUnits = null;
@@ -467,7 +508,8 @@ private NameEnvironmentAnswer findClass(String qualifiedTypeName, char[] typeNam
 		if (modulePathEntry instanceof ModulePathEntry) {
 			relevantLocations = ((ModulePathEntry) modulePathEntry).getClasspathLocations();
 		} else if (modulePathEntry instanceof ClasspathLocation) {
-			return ((ClasspathLocation) modulePathEntry).findClass(typeName, qPackageName, moduleName, qBinaryFileName, false);
+			return ((ClasspathLocation) modulePathEntry).findClass(typeName, qPackageName, moduleName, qBinaryFileName, false,
+																	null/*module already checked*/);
 		} else {
 			return null;
 		}
@@ -479,7 +521,8 @@ private NameEnvironmentAnswer findClass(String qualifiedTypeName, char[] typeNam
 		if (!strategy.matches(classpathLocation, ClasspathLocation::hasModule)) {
 			continue;
 		}
-		NameEnvironmentAnswer answer = classpathLocation.findClass(binaryFileName, qPackageName, moduleName, qBinaryFileName, false);
+		NameEnvironmentAnswer answer = classpathLocation.findClass(binaryFileName, qPackageName, moduleName, qBinaryFileName, false,
+																	this.modulePathEntries != null ? this.modulePathEntries::containsKey : null);
 		if (answer != null) {
 			char[] answerMod = answer.moduleName();
 			if (answerMod != null && this.modulePathEntries != null) {
@@ -550,7 +593,8 @@ public char[][] getModulesDeclaringPackage(char[][] parentPackageName, char[] na
 		default:
 			if (this.modulePathEntries != null) {
 				names = CharOperation.NO_CHAR_CHAR;
-				for (IModulePathEntry modulePathEntry : this.modulePathEntries.values()) {
+				Collection<IModulePathEntry> entries = new HashSet<>(this.modulePathEntries.values());
+				for (IModulePathEntry modulePathEntry : entries) {
 					char[][] declaringModules = modulePathEntry.getModulesDeclaringPackage(pkgName, modName);
 					if (declaringModules != null)
 						names = CharOperation.arrayConcat(names, declaringModules);
