@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -567,27 +570,35 @@ public void indexAll(IProject project) {
 	this.indexer.makeDirty(project);
 	if (JavaCore.getPlugin() == null) return;
 
-	// Also request indexing of binaries on the classpath
-	// determine the new children
 	try {
-		JavaModel model = JavaModelManager.getJavaModelManager().getJavaModel();
-		JavaProject javaProject = (JavaProject) model.getJavaProject(project);
-		// only consider immediate libraries - each project will do the same
-		// NOTE: force to resolve CP variables before calling indexer - 19303, so that initializers
-		// will be run in the current thread.
-		IClasspathEntry[] entries = javaProject.getResolvedClasspath();
-		for (int i = 0; i < entries.length; i++) {
-			IClasspathEntry entry= entries[i];
-			if (entry.getEntryKind() == IClasspathEntry.CPE_LIBRARY)
-				indexLibrary(entry.getPath(), project, ((ClasspathEntry)entry).getLibraryIndexLocation());
-		}
-	} catch(JavaModelException e){ // cannot retrieve classpath info
-	}
+		// Disable index manager to avoid synchronization lock contention when adding new index requests to the queue.
+		disable();
 
-	// check if the same request is not already in the queue
-	IndexRequest request = new IndexAllProject(project, this);
-	if (!isJobWaiting(request))
-		request(request);
+		// Also request indexing of binaries on the classpath
+		// determine the new children
+		try {
+			JavaModel model = JavaModelManager.getJavaModelManager().getJavaModel();
+			JavaProject javaProject = (JavaProject) model.getJavaProject(project);
+			// only consider immediate libraries - each project will do the same
+			// NOTE: force to resolve CP variables before calling indexer - 19303, so that initializers
+			// will be run in the current thread.
+			IClasspathEntry[] entries = javaProject.getResolvedClasspath();
+			for (int i = 0; i < entries.length; i++) {
+				IClasspathEntry entry= entries[i];
+				if (entry.getEntryKind() == IClasspathEntry.CPE_LIBRARY)
+					indexLibrary(entry.getPath(), project, ((ClasspathEntry)entry).getLibraryIndexLocation());
+			}
+		} catch(JavaModelException e){ // cannot retrieve classpath info
+		}
+
+		// check if the same request is not already in the queue
+		IndexRequest request = new IndexAllProject(project, this);
+		if (!isJobWaiting(request))
+			request(request);
+	} finally {
+		// Enable index manager after adding all new index requests to the queue.
+		enable();
+	}
 }
 public void indexLibrary(IPath path, IProject requestingProject, URL indexURL) {
 	this.indexLibrary(path, requestingProject, indexURL, false);
@@ -684,6 +695,7 @@ public synchronized void jobWasCancelled(IPath containerPath) {
  * Advance to the next available job, once the current one has been completed.
  * Note: clients awaiting until the job count is zero are still waiting at this point.
  */
+@Override
 protected synchronized void moveToNextJob() {
 	// remember that one job was executed, and we will need to save indexes at some point
 	this.needToSave = true;
@@ -692,12 +704,14 @@ protected synchronized void moveToNextJob() {
 /**
  * No more job awaiting.
  */
+@Override
 protected void notifyIdle(long idlingTime){
 	if (idlingTime > 1000 && this.needToSave) saveIndexes();
 }
 /**
  * Name of the background process
  */
+@Override
 public String processName(){
 	return Messages.process_name;
 }
@@ -902,6 +916,7 @@ public void removeSourceFolderFromIndex(JavaProject javaProject, IPath sourceFol
 /**
  * Flush current state
  */
+@Override
 public void reset() {
 	super.reset();
 	synchronized (this) {
@@ -1032,8 +1047,11 @@ public void saveIndexes() {
 }
 public void scheduleDocumentIndexing(final SearchDocument searchDocument, IPath container, final IndexLocation indexLocation, final SearchParticipant searchParticipant) {
 	IPath targetLocation = JavaIndex.getLocationForPath(new Path(searchDocument.getPath()));
-	this.indexer.makeDirty(targetLocation);
+	if (targetLocation != null) {
+		this.indexer.makeDirty(targetLocation);
+	}
 	request(new IndexRequest(container, this) {
+		@Override
 		public boolean execute(IProgressMonitor progressMonitor) {
 			if (this.isCancelled || progressMonitor != null && progressMonitor.isCanceled()) return true;
 
@@ -1054,6 +1072,7 @@ public void scheduleDocumentIndexing(final SearchDocument searchDocument, IPath
 			}
 			return true;
 		}
+		@Override
 		public String toString() {
 			return "indexing " + searchDocument.getPath(); //$NON-NLS-1$
 		}
@@ -1064,6 +1083,7 @@ public void scheduleDocumentIndexing(final SearchDocument searchDocument, IPath
 	});
 }
 
+@Override
 public String toString() {
 	StringBuffer buffer = new StringBuffer(10);
 	buffer.append(super.toString());
